/*
 * URIResolverImpl.java <http://web.vee.net/projects/uriresolver/>
 * Copyright (c) 2001 Michael Gratton <mike@vee.net>. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Michael Gratton <mike@vee.net>."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

import java.io.*;
import java.util.StringTokenizer;

import javax.xml.transform.*;
import javax.xml.transform.stream.*;

import org.xmldb.api.*;
import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;

/**
 * <p>An example URIResolver for XMLDB-compliant databases. Note, don't
 * use this in a production system, rather use it as an example for
 * writing a decent resolver of your own.</p>
 * 
 * <p>This resolver accepts URIs with a 'xmldb' schema and an empty
 * authority [host], it does not resolve relative hrefs against
 * any given base, so all URIs handed to it must be absolute.
 * For example: <code>xmldb:///foo/bar.xml</code></p>
 */
public class URIResolverImpl implements URIResolver {

    private Collection mRoot;

    /**
     * Constructs a new URIResolver.
     * @param aRootCollection a non-null Collection which
     * will be used as the root collection when resolving URIs.
     */
    public URIResolverImpl(Collection aRootCollection) {
	mRoot = aRootCollection;
    }

    /**
     * Resolves a href to an XML document.
     */
    public Source resolve(String aHref, String aBase)
	throws TransformerException {

	// only accept 'xmldb://' URIs
	if (aHref == null || !aHref.startsWith("xmldb://"))
	    return null;

	// we can't resolve to a Collection (indicated by a trailing '/')
	if (aHref.charAt(aHref.length() - 1) == '/')
	    return null;

	StringTokenizer tokens = new StringTokenizer(aHref.substring(9), "/");
	if (!tokens.hasMoreTokens())
	    return null; // nothing after the 'xmldb://'

	// recurse down the collection hierachy, looking for our target
	String stepName;
	Collection step = mRoot;
	for (;;) {
	    stepName = tokens.nextToken();

	    try {
		if (tokens.hasMoreTokens()) {
		    // looking for a collection
		    step = step.getChildCollection(stepName);
		    
		    if (step == null)
			return null;
		}
		else {
		    // looking for a resource
		    Resource resource = step.getResource(stepName);
		    
		    if (resource == null || !(resource instanceof XMLResource))
			return null;

		    // found it, return as a Source
		    XMLResource xml = (XMLResource)resource;
		    String content = (String)xml.getContent();
		    StringReader xmlReader = new StringReader(content);

		    return new StreamSource(xmlReader);
		}
	    }
	    catch (XMLDBException xde) {
		throw new TransformerException
		    ("XMLDB Error when resolving href.", xde);
	    }
	}
    }

    /**
     * <p>Self test / demo. Requires five parameters (in order):</p>
     * <ul>
     *   <li>XMLDB database class name</li>
     *   <li>XMLDB root collection URI</li>
     *   <li>source XML file</li>
     *   <li>transform XSLT file</li>
     *   <li>result XML file</li>
     * </ul>
     */
    public static void main(String[] args) {

	try {
	    // for teh sake of clarity..
	    String dbClass = args[0];
	    String rootURI = args[1];
	    String xmlFile = args[2];
	    String xsltFile = args[3];
	    String resultFile = args[4];

	    // set up our xmldb objects
	    Database database = (Database)Class.forName(dbClass).newInstance();
	    DatabaseManager.registerDatabase(database);
	    
	    Collection collection  = DatabaseManager.getCollection(rootURI);

	    // construct our resolver
	    URIResolverImpl resolver = new URIResolverImpl(collection);

	    // set up a transformer
	    TransformerFactory factory = TransformerFactory.newInstance();
	    factory.setURIResolver(resolver);

	    Transformer transformer = factory.newTransformer(new StreamSource(xsltFile));
	    transformer.setURIResolver(resolver);

	    FileReader in = new FileReader(xmlFile);
	    FileWriter out = new FileWriter(resultFile);

	    // do the transform
	    transformer.transform(new StreamSource(in), new StreamResult(out));
	}
	catch (Exception e) {
	    e.printStackTrace();
	    System.err.println("Usage: URIResolverImpl [db class] [root] [xml] [xslt] [result]");
	}
    }

}
